# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/6/10 10:41 AM
# LAST MODIFIED ON:
# AIM:

import unittest
from sentence_spliter_old.logic_graph_en import long_cuter_en
from sentence_spliter_old.automata.sequence import EnSequence
from sentence_spliter_old.automata.state_machine import StateMachine


class test_spliter(unittest.TestCase):
    def test_bug_2021_5_21(self):
        paragraph = '''Yes because he never did a thing like that before as ask to get his breakfast in bed with a couple of eggs since the City Arms hotel when he used to be pretending to be laid up with a sick voice doing his highness to make himself interesting for that old faggot Mrs Riordan that he thought he had a great leg of and she never left us a farthing all for masses for herself and her soul greatest miser ever was actually afraid to lay out 4d for her methylated spirit telling me all her ailments she had too much old chat in her about politics and earthquakes and the end of the world let us have a bit of fun first God help the world if all the women were her sort down on bathingsuits and lownecks of course nobody wanted her to wear them I suppose she was pious because no man would look at her twice I hope Ill never be like her a wonder she didnt want us to cover our faces but she was a welleducated woman certainly and her gabby talk about Mr Riordan here and Mr Riordan there I suppose he was glad to get shut of her and her dog smelling my fur and always edging to get up under my petticoats especially then still I like that in him polite to old women like that and waiters and beggars too hes not proud out of nothing but not always if ever he got anything really serious the matter with him its much better for them to go into a hospital where everything is clean but I suppose Id have to dring it into him for a month yes and then wed have a hospital nurse next thing on the carpet have him staying there till they throw him out or a nun maybe like the smutty photo he has shes as much a nun as Im not yes because theyre so weak and puling when theyre sick they want a woman to get well if his nose bleeds youd think it was O tragic and that dyinglooking one off the south circular when he sprained his foot at the choir party at the sugarloaf Mountain the day I wore that dress Miss Stack bringing him flowers the worst old ones she could find at the bottom of the basket anything at all to get into a mans bedroom with her old maids voice trying to imagine he was dying on account of her to never see thy face again though he looked more like a man with his beard a bit grown in the bed father was the same besides I hate bandaging and dosing when he cut his toe with the razor paring his corns afraid hed get bloodpoisoning but if it was a thing I was sick then wed see what attention only of course the woman hides it not to give all the trouble they do yes he came somewhere Im sure by his appetite anyway love its not or hed be off his feed thinking of her so either it was one of those night women if it was down there he was really and the hotel story he made up a pack of lies to hide it planning it Hynes kept me who did I meet ah yes I met do you remember Menton and who else who let me see that big babbyface I saw him and he not long married flirting with a young girl at Pooles Myriorama and turned my back on him when he slinked out looking quite conscious what harm but he had the impudence to make up to me one time well done to him mouth almighty and his boiled eyes of all the big stupoes I ever met and thats called a solicitor only for I hate having a long wrangle in bed or else if its not that its some little bitch or other he got in with somewhere or picked up on the sly if they only knew him as well as I do yes because the day before yesterday he was scribbling something a letter when I came into the front room to show him Dignams death in the paper as if something told me and he covered it up with the blottingpaper pretending to be thinking about business so very probably that was it to somebody who thinks she has a softy in him because all men get a bit like that at his age especially getting on to forty he is now so as to wheedle any money she can out of him no fool like an old fool and then the usual kissing my bottom was to hide it not that I care two straws now who he does it with or knew before that way though Id like to find out so long as I dont have the two of them under my nose all the time like that slut that Mary we had in Ontario terrace padding out her false bottom to excite him bad enough to get the smell of those painted women off him once or twice I had a suspicion by getting him to come near me when I found the long hair on his coat without that one when I went into the kitchen pretending he was drinking water 1 woman is not enough for them it was all his fault of course ruining servants then proposing that she could eat at our table on Christmas day if you please O no thank you not in my house stealing my potatoes and the oysters 2/6 per doz going out to see her aunt if you please common robbery so it was but I was sure he had something on with that one it takes me to find out a thing like that he said you have no proof it was her proof O yes her aunt was very fond of oysters but I told her what I thought of her suggesting me to go out to be alone with her I wouldnt lower myself to spy on them the garters I found in her room the Friday she was out that was enough for me a little bit too much her face swelled up on her with temper when I gave her her weeks notice I saw to that better do without them altogether do out the rooms myself quicker only for the damn cooking and throwing out the dirt I gave it to him anyhow either she or me leaves the house I couldnt even touch him if I thought he was with a dirty barefaced liar and sloven like that one denying it up to my face and singing about the place in the W C too because she knew she was too well off yes because he couldnt possibly do without it that long so he must do it somewhere and the last time he came on my bottom when was it the night Boylan gave my hand a great squeeze going along by the Tolka in my hand there steals another I just pressed the back of his like that with my thumb to squeeze back singing the young May moon shes beaming love because he has an idea about him and me hes not such a fool he said Im dining out and going to the Gaiety though Im not going to give him the satisfaction in any case God knows hes a change in a way not to be always and ever wearing the same old hat unless I paid some nicelooking boy to do it since I cant do it myself a young boy would like me Id confuse him a little alone with him if we were Id let him see my garters the new ones and make him turn red looking at him seduce him I know what boys feel with that down on their cheek doing that frigging drawing out the thing by the hour question and answer would you do this that and the other with the coalman yes with a bishop yes I would because I told him about some dean or bishop was sitting beside me in the jews temples gardens when I was knitting that woollen thing a stranger to Dublin what place was it and so on about the monuments and he tired me out with statues encouraging him making him worse than he is who is in your mind now tell me who are you thinking of who is it tell me his name who tell me who the german Emperor is it yes imagine Im him think of him can you feel him trying to make a whore of me what he never will he ought to give it up now at this age of his life simply ruination for any woman and no satisfaction in it pretending to like it till he comes and then finish it off myself anyway and it makes your lips pale anyhow its done now once and for all with all the talk of the world about it people make its only the first time after that its just the ordinary do it and think no more about it why cant you kiss a man without going and marrying him first you sometimes love to wildly when you feel that way so nice all over you you cant help yourself I wish some man or other would take me sometime when hes there and kiss me in his arms theres nothing like a kiss long and hot down to your soul almost paralyses you then I hate that confession when I used to go to Father Corrigan he touched me father and what harm if he did where and I said on the canal bank like a fool but whereabouts on your person my child on the leg behind high up was it yes rather high up was it where you sit down yes O Lord couldnt he say bottom right out and have done with it what has that got to do with it and did you whatever way he put it I forget no father and I always think of the real father what did he want to know for when I already confessed it to God he had a nice fat hand the palm moist always I wouldnt mind feeling it neither would he Id say by the bullneck in his horsecollar I wonder did he know me in the box I could see his face he couldnt see mine of course hed never turn or let on still his eyes were red when his father died theyre lost for a woman of course must be terrible when a man cries let alone them Id like to be embraced by one in his vestments and the smell of incense off him like the pope besides theres no danger with a priest if youre married hes too careful about himself then give something to H H the pope for a penance I wonder was he satisfied with me one thing I didnt like his slapping me behind going away so familiarly in the hall though I laughed Im not a horse or an ass am I I suppose he was thinking of his fathers I wonder is he awake thinking of me or dreaming am I in it who gave him that flower he said he bought he smelt of some kind of drink not whisky or stout or perhaps the sweety kind of paste they stick their bills up with some liqueur Id like to sip those richlooking green and yellow expensive drinks those stagedoor johnnies drink with the opera hats I tasted once with my finger dipped out of that American that had the squirrel talking stamps with father he had all he could do to keep himself from falling asleep after the last time after we took the port and potted meat it had a fine salty taste yes because I felt lovely and tired myself and fell asleep as sound as a top the moment I popped straight into bed till that thunder woke me up God be merciful to us I thought the heavens were coming down about us to punish us when I blessed myself and said a Hail Mary like those awful thunderbolts in Gibraltar as if the world was coming to an end and then they come and tell you theres no God what could you do if it was running and rushing about nothing only make an act of contrition the candle I lit that evening in Whitefriars street chapel for the month of May see it brought its luck though hed scoff if he heard because he never goes to church mass or meeting he says your soul you have no soul inside only grey matter because he doesnt know what it is to have one yes when I lit the lamp because he must have come 3 or 4 times with that tremendous big red brute of a thing he has I thought the vein or whatever the dickens they call it was going to burst though his nose is not so big after I took off all my things with the blinds down after my hours dressing and perfuming and combing it like iron or some kind of a thick crowbar standing all the time he must have eaten oysters I think a few dozen he was in great singing voice no I never in all my life felt anyone had one the size of that to make you feel full up he must have eaten a whole sheep after whats the idea making us like that with a big hole in the middle of us or like a Stallion driving it up into you because thats all they want out of you with that determined vicious look in his eye I had to halfshut my eyes still he hasnt such a tremendous amount of spunk in him when I made him pull out and do it on me considering how big it is so much the better in case any of it wasnt washed out properly the last time I let him finish it in me nice invention they made for women for him to get all the pleasure but if someone gave them a touch of it themselves theyd know what I went through with Milly nobody would believe cutting her teeth too and Mina Purefoys husband give us a swing out of your whiskers filling her up with a child or twins once a year as regular as the clock always with a smell of children off her the one they called budgers or something like a nigger with a shock of hair on it Jesusjack the child is a black the last time I was there a squad of them falling over one another and bawling you couldnt hear your ears supposed to be healthy not satisfied till they have us swollen out like elephants or I dont know what supposing I risked having another not off him though still if he was married Im sure hed have a fine strong child but I dont know Poldy has more spunk in him yes thatd be awfully jolly I suppose it was meeting Josie Powell and the funeral and thinking about me and Boylan set him off well he can think what he likes now if thatll do him any good I know they were spooning a bit when I came on the scene he was dancing and sitting out with her the night of Georgina Simpsons housewarming and then he wanted to ram it down my neck it was on account of not liking to see her a wallflower that was why we had the standup row over politics he began it not me when he said about Our Lord being a carpenter at last he made me cry of course a woman is so sensitive about everything I was fuming with myself after for giving in only for I knew he was gone on me and the first socialist he said He was he annoyed me so much I couldnt put him into a temper still he knows a lot of mixedup things especially about the body and the inside I often wanted to study up that myself what we have inside us in that family physician I could always hear his voice talking when the room was crowded and watch him after that I pretended I had a coolness on with her over him because he used to be a bit on the jealous side whenever he asked who are you going to and I said over to Floey and he made me the present of Byrons poems and the three pairs of gloves so that finished that I could quite easily get him to make it up any time I know how Id even supposing he got in with her again and was going out to see her somewhere Id know if he refused to eat the onions I know plenty of ways ask him to tuck down the collar of my blouse or touch him with my veil and gloves on going out 1 kiss then would send them all spinning however alright well see then let him go to her she of course would only be too delighted to pretend shes mad in love with him that I wouldnt so much mind Id just go to her and ask her do you love him and look her square in the eyes she couldnt fool me but he might imagine he was and make a declaration to her with his plabbery kind of a manner like he did to me though I had the devils own job to get it out of him though I liked him for that it showed he could hold in and wasnt to be got for the asking he was on the pop of asking me too the night in the kitchen I was rolling the potato cake theres something I want to say to you only for I put him off letting on I was in a temper with my hands and arms full of pasty flour in any case I let out too much the night before talking of dreams so I didnt want to let him know more than was good for him she used to be always embracing me Josie whenever he was there meaning him of course glauming me over and when I said I washed up and down as far as possible asking me and did you wash possible the women are always egging on to that putting it on thick when hes there they know by his sly eye blinking a bit putting on the indifferent when they come out with something the kind he is what spoils him I dont wonder in the least because he was very handsome at that time trying to look like Lord Byron I said I liked though he was too beautiful for a man and he was a little before we got engaged afterwards though she didnt like it so much the day I was in fits of laughing with the giggles I couldnt stop about all my hairpins falling out one after another with the mass of hair I had youre always in great humour she said yes because it grigged her because she knew what it meant because I used to tell her a good bit of what went on between us not all but just enough to make her mouth water but that wasnt my fault she didnt darken the door much after we were married I wonder what shes got like now after living with that dotty husband of hers she had her face beginning to look drawn and run down the last time I saw her she must have been just after a row with him because I saw on the moment she was edging to draw down a conversation about husbands and talk about him to run him down what was it she told me O yes that sometimes he used to go to bed with his muddy boots on when the maggot takes him just imagine having to get into bed with a thing like that that might murder you any moment what a man well its not the one way everyone goes mad Poldy anyhow whatever he does always wipes his feet on the mat when he comes in wet or shine and always blacks his own boots too and he always takes off his hat when he comes up in the street like then and now hes going about in his slippers to look for £ 10000 for a postcard U p up O sweetheart May wouldnt a thing like that simply bore you stiff to extinction actually too stupid even to take his boots off now what could you make of a man like that Id rather die 20 times over than marry another of their sex of course hed never find another woman like me to put up with him the way I do know me come sleep with me yes and he knows that too at the bottom of his heart take that Mrs Maybrick that poisoned her husband for what I wonder in love with some other man yes it was found out on her wasnt she the downright villain to go and do a thing like that of course some men can be dreadfully aggravating drive you mad and always the worst word in the world what do they ask us to marry them for if were so bad as all that comes to yes because they cant get on without us white Arsenic she put in his tea off flypaper wasnt it I wonder why they call it that if I asked him hed say its from the Greek leave us as wise as we were before she must have been madly in love with the other fellow to run the chance of being hanged O she didnt care if that was her nature what could she do besides theyre not brutes enough to go and hang a woman surely are they'''

        __long_machine_en = StateMachine(long_cuter_en(min_len=6, max_len=30))
        m_input = EnSequence(paragraph)
        __long_machine_en.run(m_input, verbose=True)
        acutal = m_input.sentence_list()
        expect = [paragraph]
        self.assertEqual(expect, acutal)




# -*- coding:utf-8 -*-
import re
import os
import sys


class Spliter:

    def __init__(self, option):
        """
        :param option: 决定是中文还是英文
        """
        self.option = option
        self.end_flags = ['?', '!', '.', '…', '？', '！', '。', '……']  # 句子结束符号，包括中文和英文的
        self.sentences = []
        self.flag = 0
        self.flag_quote = 0
        self.tmp_char = ''
        # -- parameter -- #

    # -- filter -- #

    # -- sentences segmentation -- #
    def cut_to_sentences(self, paragraph, verbose: bool = True):
        """
        切分段落
        :param verbose:
        :param paragraph:
        :return: 切分好的list
        """
        self.sentences = []
        self.tmp_char = ''
        if len(paragraph) == 0:
            return []
        total_len = len(paragraph)
        for index, char in enumerate(paragraph):
            if verbose:
                percent = float(index) / total_len * 100.0
                sys.stdout.write('\rprocess sentence cut : {0:.2g}%'.format(percent))
            self._do_cut(char, index, paragraph)
        outputs = []
        outputs.extend(sente.replace('\n', '') for sente in self.sentences if len(sente) > 1)  # 去掉空值和长度为1的
        return outputs

    def _do_cut(self, char, index, paragraph):
        """
        切分的状态机，再封装各个情况，每个if能拆解为函数
        :param index:
        :param char:
        :param paragraph:
        """
        self.tmp_char += char
        if self._last_char(index, paragraph) is True:  # 判断是否已经到了段落最后一个字符
            return
        if char == '”' or char == '"':
            self.flag = 0
            if paragraph[index + 1] == '，' or paragraph[index + 1] == ',':
                pass
            else:
                self.sentences.append(self.tmp_char)
                self.tmp_char = ''

        if char == '“' or char == '"' and self.flag_quote == 0:
            start_char = self.quota_begin()
            end_char = self.quota_end()
            if self.match_quota_sentences(start_char, end_char, paragraph[index:]) is True:
                self.flag = 1
                self.flag_quote = 1
        if self._end_char(char) == True and self.flag == 0:  # 判断此字符是否为结束符号
            if self._demical_char(char, index, paragraph) is True:  # 判断是否是小数
                return
            elif paragraph[index + 1] not in self.end_flags:
                self.sentences.append(self.tmp_char)
                self.tmp_char = ''

    def match_quota_sentences(self, start_char, end_char, sentence):
        """
        匹配有引号的句子
        :param start_char:
        :param end_char:
        :param sentence:
        :return: True/False 是否匹配到有引号的句子
        """
        pattern_str = r'%s(.+?)%s' % (start_char, end_char)
        p = re.compile(pattern_str, re.IGNORECASE)
        m = re.match(p, sentence)
        if m:
            return True
        else:
            return False

    def quota_begin(self):
        """
        引号开始
        :return:start_char
        """
        if self.option == 1:
            start_char = '“'
        else:
            start_char = '"'
        return start_char

    def quota_end(self):
        """
        引号结束
        :return:end_char
        """
        if self.option == 1:
            end_char = '”'
        else:
            end_char = '"'
        return end_char

    def _last_char(self, index, paragraph):
        """
        是否是段落最后一个字符
        :param index:
        :param paragraph:
        :return: True/False
        """
        if (index + 1) == len(paragraph):
            self.sentences.append(self.tmp_char)
            return True
        else:
            return False

    def _end_char(self, char):
        """
        是否是划分句子的字符
        :param char:
        :return: True/False
        """
        if char in self.end_flags:
            return True
        else:
            return False

    def _demical_char(self, char, index, paragraph):
        """
        是否是小数
        :param char:
        :param index:
        :param paragraph:
        :return: True/False
        """
        if char == '.' and paragraph[index - 1].isdigit() and paragraph[index + 1].isdigit():
            return True
        else:
            return False
